import React, { Component } from 'react';
import {
  StyleSheet,
  Text,
  View,
  Modal,
  Slider,
  Platform,
  TouchableOpacity
} from 'react-native';
import tinycolor from 'tinycolor2';
import type from './type';

const Gradient = ({
  step,
  maximumValue,
  value,
  getColor,
  style,
  onValueChange
}) => {
  const rows = [];
  for (let i = 0; i <= maximumValue / step; i++) {
    rows.push(
      <View
        key={i}
        style={[
          styles.colorStep,
          {
            backgroundColor: getColor(i * step)
          }
        ]}
      />
    );
  }
  return (
    <View style={[styles.slider, style]}>
      <View style={styles.colorBar}>{rows}</View>
      <Slider
        style={styles.slider}
        value={value}
        step={step}
        maximumValue={maximumValue}
        onValueChange={onValueChange}
        minimumTrackTintColor="transparent"
        maximumTrackTintColor="transparent"
        thumbTintColor="black"
      />
    </View>
  );
};

const modes = {
  hex: {
    getString: color => tinycolor(color).toHexString(),
    label: 'HEX'
  },
  hsl: {
    getString: color => tinycolor(color).toHslString(),
    label: 'HSL'
  },
  hsv: {
    getString: color => tinycolor(color).toHsvString(),
    label: 'HSV'
  },
  rgb: {
    getString: color => tinycolor(color).toRgbString(),
    label: 'RGB'
  }
};

export class ColorPicker extends Component {
  state = {
    color: { h: 170, s: 0.42, v: 0.76 },
    mode: 'hex'
  };

  render() {
    const { visible, onRequestClose, swatches } = this.props;
    return (
      <Modal
        animationType="slide"
        transparent={false}
        visible={visible}
        onRequestClose={onRequestClose}
      >
        <View style={styles.container}>
          <View style={styles.header}>
            <TouchableOpacity onPress={onRequestClose}>
              <Text style={type.bodyEm}>Cancel</Text>
            </TouchableOpacity>
            <TouchableOpacity onPress={onRequestClose}>
              <Text style={type.bodyEm}>Done</Text>
            </TouchableOpacity>
          </View>
          <View style={styles.content}>
            <View
              style={{
                backgroundColor: tinycolor(this.state.color).toHexString(),
                flex: 1,
                flexDirection: 'column',
                alignItems: 'flex-end',
                justifyContent: 'flex-end',
                paddingVertical: 8,
                paddingHorizontal: 12
              }}
            >
              <Text style={[type.bodyEm, { color: 'white' }]}>LIGHT TEXT</Text>
              <Text style={[type.bodyEm, { marginTop: 6 }]}>DARK TEXT</Text>
            </View>
            <View style={styles.colorString}>
              <Text style={styles.colorStringText}>
                {modes[this.state.mode].getString(this.state.color)}
              </Text>
            </View>
            <View style={styles.modesRow}>
              {Object.keys(modes).map(key => (
                <View
                  key={key}
                  style={[
                    styles.mode,
                    this.state.mode === key && styles.modeActive
                  ]}
                >
                  <TouchableOpacity
                    onPress={() => this.setState({ mode: key })}
                  >
                    <Text
                      style={[
                        styles.modeText,
                        this.state.mode === key && styles.modeTextActive
                      ]}
                    >
                      {modes[key].label}
                    </Text>
                  </TouchableOpacity>
                </View>
              ))}
            </View>
            <View style={styles.sliders}>
              <Gradient
                style={styles.sliderRow}
                step={12}
                maximumValue={359}
                value={this.state.color.h}
                getColor={i => tinycolor({ s: 1, v: 1, h: i }).toHexString()}
                onValueChange={h =>
                  this.setState({ color: { ...this.state.color, h } })}
              />
              <Gradient
                style={styles.sliderRow}
                step={0.05}
                maximumValue={1}
                value={this.state.color.s}
                getColor={i =>
                  tinycolor({ ...this.state.color, s: i }).toHexString()}
                onValueChange={s =>
                  this.setState({ color: { ...this.state.color, s } })}
              />
              <Gradient
                style={styles.sliderRow}
                step={0.05}
                maximumValue={1}
                value={this.state.color.v}
                getColor={i =>
                  tinycolor({ ...this.state.color, v: i }).toHexString()}
                onValueChange={v =>
                  this.setState({ color: { ...this.state.color, v } })}
              />
            </View>
            <Text style={styles.swatchesText}>RECENTS</Text>
            <View style={styles.swatchesContainer}>
              {swatches.map((swatch, index) => (
                <TouchableOpacity
                  key={swatch}
                  style={[
                    styles.swatch,
                    {
                      backgroundColor: swatch,
                      marginRight: index < swatches.length - 1 ? 16 : 0
                    }
                  ]}
                  onPress={() =>
                    this.setState({ color: tinycolor(swatch).toHsv() })}
                />
              ))}
            </View>
          </View>
        </View>
      </Modal>
    );
  }
}

export default ColorPicker;

const styles = StyleSheet.create({
  container: {
    flex: 1,
    backgroundColor: '#fff',
    alignItems: 'stretch',
    justifyContent: 'center',
    marginTop: Platform.OS === 'ios' ? 20 : 0
  },
  header: {
    backgroundColor: '#fff',
    flexDirection: 'row',
    alignItems: 'center',
    justifyContent: 'space-between',
    height: 64,
    marginHorizontal: 16
  },
  content: {
    flex: 1,
    marginHorizontal: 16
  },
  modesRow: {
    marginTop: 12,
    flexDirection: 'row',
    alignItems: 'center',
    justifyContent: 'flex-start'
  },
  mode: {
    paddingHorizontal: 6,
    paddingVertical: 1,
    marginRight: 16
  },
  modeActive: {
    backgroundColor: 'black',
    borderRadius: 3
  },
  modeText: {
    ...type.footnote
  },
  modeTextActive: {
    color: 'white'
  },
  sliders: {
    marginTop: 16
  },
  sliderRow: {
    marginTop: 12
  },
  slider: {
    flexDirection: 'column',
    justifyContent: 'center',
    alignItems: 'stretch',
    height: 32
  },
  colorBar: {
    position: 'absolute', //TODO Review
    flexDirection: 'row',
    alignItems: 'stretch',
    minHeight: 6
  },
  colorStep: {
    flex: 1
  },
  colorString: {
    marginTop: 32,
    borderBottomWidth: 2,
    borderColor: '#DDDDDD'
  },
  colorStringText: {
    lineHeight: 24,
    fontSize: 20,
    ...Platform.select({
      android: {
        fontFamily: 'monospace'
      },
      ios: {
        fontFamily: 'Courier New',
        fontWeight: '600',
        letterSpacing: 0.75
      }
    })
  },
  swatchesText: {
    marginTop: 16,
    ...type.footnote,
    color: '#555'
  },
  swatchesContainer: {
    marginTop: 12,
    marginBottom: 24,
    flexDirection: 'row',
    alignItems: 'flex-start',
    justifyContent: 'flex-start'
  },
  swatch: {
    flex: 1,
    aspectRatio: 1,
    borderRadius: 3
  }
});
